# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Manage entries to remind to users'''

# standard libraires imports
import os
import os.path
import sys

def getlastblogentry(options):
    '''Return the last blog entry'''

    lastblogentry = ''
    newlist = []
    nextmessage = False
    pathtolist = options['path_to_list']
    # read the list of blog entries
    with open(pathtolist) as lstc:
        listofentries = lstc.readlines()
    for line in listofentries:
        elements = line.split()
        if len(elements) < 3:
            sys.exit('The list of messages does not seem valid. Please format it correcty before retrying')
        else:
            # extract the current line from the file of the list of blog entries
            status = elements[0]
            # detect if the line should be used
            if status == 'o' and not nextmessage:
                nextmessage = True
                lastblogentry = line[2:].rstrip('\n')
                newlist.append(''.join(['x', line[1:]]))
            # configure the next line to be usded
            elif status == 'x' and nextmessage:
                newlist.append(''.join(['o', line[1:]]))
                nextmessage = False
            else:
                newlist.append(line)
    # checking if entry was found on the last line of the file
    if nextmessage:
        newlist[0] = 'o' + newlist[0][1:]
    # rewrite the modified file
    with open(pathtolist, 'w') as lstc:
        lstc.writelines(newlist)
    return lastblogentry



# -*- coding: utf-8 -*-
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# CLI parsing
'''CLI parsing'''

# standard library imports
from argparse import ArgumentParser
import glob
import logging
import os.path
import sys

__version__ = '0.5'

def cliparse():
    '''Parse the command line to get options'''
    epilog = 'For more information: https://remindr.readhthedocs.org'
    description = 'Send toots to remind people about blog entries'
    parser = ArgumentParser(prog='remindr',
                            description=description,
                            epilog=epilog)
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('-c', '--config',
                        default=[os.path.join(os.getenv('XDG_CONFIG_HOME', '~/.config'),
                                              'remindr.ini')],
                        nargs='+',
                        dest="config",
                        help='Location of config file (default: %(default)s)',
                        metavar='FILE')
    parser.add_argument('-n', '--dry-run', dest='dryrun',
                        action='store_true', default=False,
                        help='Do not actually post toots')
    parser.add_argument('-v', '--verbose', '--info', dest='log_level',
                        action='store_const', const='info', default='warning',
                        help='enable informative (verbose) output, work on log level INFO')
    parser.add_argument('-d', '--debug', dest='log_level',
                        action='store_const', const='debug', default='warning',
                        help='enable debug output, work on log level DEBUG')
    levels = [i for i in logging._nameToLevel.keys()
    if (type(i) == str and i != 'NOTSET')]
    parser.add_argument('--syslog', nargs='?', default=None,
                        type=str.upper, action='store',
                        const='INFO', choices=levels,
                        help='''log to syslog facility, default: no
                        logging, INFO if --syslog is specified without
                        argument''')
    opts = parser.parse_args()
    # verify if the path to cache file is an absolute path
    # get the different config files, from a directory or from a *.ini style
    opts.config = list(map(os.path.expanduser, opts.config))
    for element in opts.config:
        if element and not os.path.exists(element):
            sys.exit('You should provide an existing path for the config file: %s' % element)
        if os.path.isdir(element):
            opts.configs = glob.glob(os.path.join(element, '*.ini'))
        else:
            # trying to glob the path
            opts.configs = glob.glob(element)
    # verify if a configuration file is provided
    if not opts.configs:
        sys.exit('no configuration file was found at the specified path(s) with the option -c')
    return opts



# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Post a reminder through a toot'''

# 3rd party libraries imports
from mastodon import Mastodon

def tootpost(clioptions, cfgoptions, toot, language):
    '''post a reminder through a toot'''
    if clioptions.dryrun:
        print('Should have tooted with visibility [{visibility}]=> {toot}'.format(visibility=cfgoptions['mastodon_toot_visibility'], toot=toot))
    else:
        mastodon = Mastodon(
            client_id=cfgoptions['mastodon_client_credentials'],
            access_token=cfgoptions['mastodon_user_credentials'],
            api_base_url=cfgoptions['mastodon_instance_url']
        )
        # if the user users image_path
        if 'path_to_image' in cfgoptions:
            mediaid = mastodon.media_post(cfgoptions['path_to_image'])
            mastodon.status_post(toot, media_ids=[mediaid], visibility=cfgoptions['mastoon_toot_visibility'])
        elif '{lang}_image_path'.format(lang=language) in cfgoptions:
            mediaid = mastodon.media_post(cfgoptions['{lang}_image_path'.format(lang=language)])
            mastodon.status_post(toot, media_ids=[mediaid], visibility=cfgoptions['mastodon_toot_visibility'])
        # if the user uses mastodon_image_path
        elif 'mastodon_image_path' in cfgoptions:
            mediaid = mastodon.media_post(cfgoptions['mastodon_image_path'])
            mastodon.status_post(toot, media_ids=[mediaid], visibility=cfgoptions['mastodon_toot_visibility'])
        # if the user users {lang}_image_path
        elif 'mastodon_{lang}_image_path'.format(lang=language) in cfgoptions:
            mediaid = mastodon.media_post(cfgoptions['mastodon_{lang}_image_path'.format(lang=language)])
            mastodon.status_post(toot, media_ids=[mediaid], visibility=cfgoptions['mastodon_toot_visibility'])
        else:
            mastodon.status_post(toot, visibility=cfgoptions['mastodon_toot_visibility'])



# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Manage entries to remind to users'''

# standard libraires imports
import os
import os.path
import sys

def formatnextmessage(options, content):
    '''Format the next message'''
    nextmessage = ''
    language = content[0:2]
    for parameter in options:
        if parameter.endswith('_prefix'):
            if parameter.startswith(language):
                nextmessage = ' '.join([options[parameter], content[3:]])
    return nextmessage, language



# -*- coding: utf-8 -*-
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the configuration file
'''Get values of the configuration file'''

# standard library imports
from configparser import SafeConfigParser, NoOptionError, NoSectionError
import logging
import os
import os.path
import socket
import sys

# 3rd party library imports
import feedparser

def cfgparse(clioptions):
    '''Parse the configurations'''
    for pathtoconfig in clioptions.configs:
        options = {}
        # read the configuration file
        config = SafeConfigParser()
        if not config.read(os.path.expanduser(pathtoconfig)):
            sys.exit('Could not read the configuration file')
        ###########################
        # 
        # the mastodon section
        # 
        ###########################
        section = 'mastodon'
        if config.has_section(section):
            ############################
            # instance_url option
            ############################
            confoption = 'instance_url'
            if config.has_option(section, confoption):
                confkey = 'mastodon_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon instance url with the "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
            ############################
            # user_credentials option
            ############################
            confoption = 'user_credentials'
            if config.has_option(section, confoption):
                confkey = 'mastodon_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon user credentials for with "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
        if config.has_section(section):
            ############################
            # client_credentials option
            ############################
            confoption = 'client_credentials'
            if config.has_option(section, confoption):
                confkey = 'mastodon_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon client credentials with the "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
            ############################
            # toot_visibility option
            ############################
            confoption = 'toot_visibility'
            if config.has_option(section, confoption):
                confkey = 'mastodon_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption, fallback='public')
            ############################
            # image_path option
            ############################
            confoption = 'image_path'
            if config.has_option(section, confoption):
                confkey = 'mastodon_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            ############################
            # {lang}_image_path option
            ############################
            for langoption in config[section]:
                if langoption.endswith('_image_path'):
                    langkey = 'mastodon_{keywithlang}'.format(keywithlang=langoption)
                    options[langkey] = config.get(section, langoption)
        ###########################
        # 
        # the twitter section
        # 
        ###########################
        section = 'twitter'
        if config.has_section(section):
            ############################
            # consumer_key option
            ############################
            confoption = 'consumer_key'
            if config.has_option(section, confoption):
                confkey = 'twitter_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon instance url with the "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
            ############################
            # consumer_secret option
            ############################
            confoption = 'consumer_secret'
            if config.has_option(section, confoption):
                confkey = 'twitter_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon instance url with the "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
            ############################
            # access_token option
            ############################
            confoption = 'access_token'
            if config.has_option(section, confoption):
                confkey = 'twitter_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon instance url with the "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
            ############################
            # access_token_secret option
            ############################
            confoption = 'access_token_secret'
            if config.has_option(section, confoption):
                confkey = 'twitter_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            else:
                sys.exit('You should define Mastodon instance url with the "{confoption}" in the [{section}] section'.format(confoption=confoption, section=section))
            ############################
            # image_path option
            ############################
            confoption = 'image_path'
            if config.has_option(section, confoption):
                confkey = 'twitter_{optionname}'.format(optionname=confoption)
                options[confkey] = config.get(section, confoption)
            ############################
            # {lang}_image_path option
            ############################
            for langoption in config[section]:
                if langoption.endswith('_image_path'):
                    langkey = 'twitter_{keywithlang}'.format(keywithlang=langoption)
                    options[langkey] = config.get(section, langoption)
        ###########################
        # 
        # the image section
        # 
        ###########################
        section = 'image'
        confoption = 'path_to_image'
        if config.has_section(section):
            if config.has_option(section, confoption):
                options['path_to_image'] = config.get(section, confoption)
            ############################
            # {lang}_image_path option
            ############################
            for langoption in config[section]:
                if langoption.endswith('_image_path'):
                    options[langoption] = config.get(section, langoption)
        ###########################
        # 
        # the entrylist section
        # 
        ###########################
        section = 'entrylist'
        if config.has_section(section):
            confoption = 'path_to_list'
            options['path_to_list'] = config.get(section, confoption)
        else:
            sys.exit('You should provide a {confoption} parameter in the [{section}] section'.format(section=section, confoption=confoption))
        ###########################
        # 
        # the prefix section
        # 
        ###########################
        section = 'prefix'
        if config.has_section(section):
            for prefixoption in config[section]:
                if prefixoption.endswith('_prefix'):
                    options[prefixoption] = config.get(section, prefixoption)
    return options



# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Post a tweet to Twitter'''

# standard libraires imports
import logging

# 3rd party libraries imports
import feedparser
import tweepy

def tweetpost(clioptions, cfgoptions, tweet, language):
    '''Post a tweet'''
    if clioptions.dryrun:
         print('Should have tweeted => {tweet}'.format(visibility=cfgoptions['mastodon_toot_visibility'], tweet=tweet))
    else:
        consumer_key = cfgoptions['twitter_consumer_key']
        consumer_secret = cfgoptions['twitter_consumer_secret']
        access_token = cfgoptions['twitter_access_token']
        access_token_secret = cfgoptions['twitter_access_token_secret']
        auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
        auth.set_access_token(access_token, access_token_secret)
        api = tweepy.API(auth)
        try:
            if 'path_to_image' in cfgoptions:
                api.update_with_media(cfgoptions['path_to_image'], tweet)
            elif '{lang}_image_path'.format(lang=language) in cfgoptions:
                api.update_with_media(cfgoptions['{lang}_image_path'.format(lang=language)], tweet)
            # if the user uses mastodon_image_path
            elif 'twitter_image_path' in cfgoptions:
                api.update_with_media(cfgoptions['twitter_image_path'], tweet)
            # if the user users {lang}_image_path
            elif 'twitter_{lang}_image_path'.format(lang=language) in cfgoptions:
                api.update_with_media(cfgoptions['twitter_{lang}_image_path'.format(lang=language)], tweet)
            else:
                api.update_status(tweet)
        except tweepy.TweepError as err:
            logging.warning('Error occurred while updating status: {err}'.format(err=err))



# Copyright 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#!/usr/bin/env python3

# Setup for Remindr
'''Setup for Remindr'''

from setuptools import setup, find_packages

CLASSIFIERS = [
    'Intended Audience :: End Users/Desktop',
    'Environment :: Console',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6'
]


setup(
    name='remindr',
    version='0.1',
    license='GNU GPL v3',
    description='Automatically send toots/tweets to remind people about the last blog entries on social networks',
    long_description='Automatically get data from a list and send messages to social networks to remind people about the most interesting blog posts you wrote',
    author = 'Carl Chenet',
    author_email = 'chaica@ohmytux.com',
    url = 'https://gitlab.com/chaica/remindr',
    classifiers=CLASSIFIERS,
    download_url='https://gitlab.com/chaica/remindr',
    packages=find_packages(),
    scripts=['scripts/remindr', 'scripts/register_remindr_app'],
    install_requires=['Mastodon.py', 'tweepy'],
)



#!/usr/bin/env python3
# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2017 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

'''Checks an RSS feed, format it, store it and posts new entries to the social networks.'''

# standard libraires imports
import codecs
import importlib
import logging
import logging.handlers
import os
import sys

# 3rd party libraries imports
import feedparser

# app libraries imports
from remindr.cliparse import cliparse
from remindr.cfgparse import cfgparse
from remindr.formatnextmessage import formatnextmessage
from remindr.getlastblogentry import getlastblogentry
from remindr.tootpost import tootpost
from remindr.tweetpost import tweetpost

class Main:
    '''Main class of Remindr'''

    def __init__(self):
        self.main()

    def setup_logging(self, options):
        if options.syslog:
            sl = logging.handlers.SysLogHandler(address='/dev/log')
            sl.setFormatter(logging.Formatter('remindr[%(process)d]: %(message)s'))
            # convert syslog argument to a numeric value
            loglevel = getattr(logging, options.syslog.upper(), None)
            if not isinstance(loglevel, int):
                raise ValueError('Invalid log level: %s' % loglevel)
            sl.setLevel(loglevel)
            logging.getLogger('').addHandler(sl)
            logging.debug('configured syslog level %s' % loglevel)
        logging.getLogger('').setLevel(logging.DEBUG)
        sh = logging.StreamHandler()
        sh.setLevel(options.log_level.upper())
        logging.getLogger('').addHandler(sh)
        logging.debug('configured stdout level %s' % sh.level)

    def main(self):
        '''The main function'''
        clioptions = cliparse()
        self.setup_logging(clioptions)
        # iterating over the different configuration files
        cfgvalues = cfgparse(clioptions)
        nextcontent = getlastblogentry(cfgvalues)
        nextmessage, language = formatnextmessage(cfgvalues, nextcontent)
        # check if mastodon instance_url is available, toot if so
        if 'mastodon_instance_url' in cfgvalues:
            tootpost(clioptions, cfgvalues, nextmessage, language)
        # check if twitter consumer_key is available, tweet if so
        if 'twitter_consumer_key' in cfgvalues:
            tweetpost(clioptions, cfgvalues, nextmessage, language)
